Package: cmdfun
Type: Package
Title: Framework for Building Interfaces to Shell Commands
Version: 1.0.2
Authors@R: person("Spencer", "Nystrom", 
    email = "nystromdev@gmail.com", 
    role = c("aut", "cre", "cph"),
    comment = c(ORCID = "0000-0003-1000-1579"))
Description: Writing interfaces to command line software is cumbersome. 
    'cmdfun' provides a framework for building function calls to seamlessly 
    interface with shell commands by allowing lazy evaluation of command line arguments. 
    'cmdfun' also provides methods for handling user-specific paths to tool installs or secrets like API keys. 
    Its focus is to equally serve package builders who wish to wrap command line software, and to help analysts stay inside 
    R when they might usually leave to execute non-R software.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: magrittr, purrr, R.utils, rlang, testthat, usethis, utils
RoxygenNote: 7.0.2
Suggests: cli, covr, knitr, processx, rmarkdown
VignetteBuilder: knitr
URL: https://snystrom.github.io/cmdfun/,
        https://github.com/snystrom/cmdfun
BugReports: https://github.com/snystrom/cmdfun
NeedsCompilation: no
Packaged: 2020-10-02 14:16:21 UTC; snystrom
Author: Spencer Nystrom [aut, cre, cph]
    (<https://orcid.org/0000-0003-1000-1579>)
Maintainer: Spencer Nystrom <nystromdev@gmail.com>
Repository: CRAN
Date/Publication: 2020-10-10 09:30:03 UTC
Built: R 4.6.0; ; 2026-04-26 17:08:50 UTC; unix
