# Generated by roxygen2: do not edit by hand

export(.check_valid_command_path)
export(.check_valid_util)
export(cmd_args_all)
export(cmd_args_dots)
export(cmd_args_named)
export(cmd_error_if_missing)
export(cmd_file_combn)
export(cmd_file_expect)
export(cmd_help_flags_similar)
export(cmd_help_flags_suggest)
export(cmd_help_parse_flags)
export(cmd_install_check)
export(cmd_install_is_valid)
export(cmd_list_drop)
export(cmd_list_drop_named)
export(cmd_list_interp)
export(cmd_list_keep)
export(cmd_list_keep_named)
export(cmd_list_to_flags)
export(cmd_path_search)
export(cmd_ui_file_exists)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(magrittr,"%T>%")
importFrom(utils,adist)
