---
title: "cmdfun"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{cmdfun}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Please see the cmdfun website for full documentation:

  * <https://snystrom.github.io/cmdfun>
  
In particular, a guide to building wrappers using cmdfun, complete with several examples, can be found here:

 * <https://snystrom.github.io/cmdfun/articles/building_wrappers.html>
  